import os
import datetime as dtm
import logging
from abc import ABC, abstractmethod
from typing import List, Optional
from decimal import Decimal

import requests
import pyqqq
from pyqqq.utils.market_schedule import get_last_trading_day
from pyqqq.datatypes import StockPosition
from pyqqq.brokerage.helper import KISConnection, EBestConnection
from pyqqq.brokerage.kis.simple import KISStockPosition
from pyqqq.brokerage.ebest.simple import EBestStockPosition
from pyqqq.utils.logger import get_logger


class BasePositionProvider(ABC):
    """백테스트 시에 보유 포지션 정보를 제공하는 추상 클래스"""

    @abstractmethod
    def get_positions(self, date: Optional[dtm.date] = None) -> List[StockPosition]:
        pass


class KISPositionProvider(BasePositionProvider):
    """한투 브로커를 사용하여 보유 포지션 정보를 제공"""

    def __init__(self):
        self.qqq = KISConnection()

    def get_positions(self, date: Optional[dtm.date] = None) -> List[KISStockPosition]:
        return self.qqq.broker_simple.get_positions()


class EBestPositionProvider(BasePositionProvider):
    """이베스트 브로커를 사용하여 보유 포지션 정보를 제공"""

    def __init__(self):
        self.qqq = EBestConnection()

    def get_positions(self, date: Optional[dtm.date] = None) -> List[EBestStockPosition]:
        return self.qqq.broker_simple.get_positions()


class ManualPositionProvider(BasePositionProvider):
    """
    백테스팅용 포지션 정보를 직접 업데이트 하는 클래스
    """
    def __init__(
        self,
        positions: List[StockPosition]
    ):
        self.positions = positions or []

    def update_positions(self, positions: List[StockPosition]):
        self.positions = positions

    def get_positions(self, date: Optional[dtm.date] = None) -> List[StockPosition]:
        return self.positions or []


class BackPositionProvider(BasePositionProvider):
    """
    DB에 저장된 포지션 정보를 가져오는 클래스
    """
    logger = get_logger("BackPositionProvider")
    logger.setLevel(logging.DEBUG)

    def __init__(
        self,
        brokerage: str = 'kis',
        api_key: str = None,
        account_no: str = None,
    ):
        self.brokerage = brokerage
        self.api_key = api_key or pyqqq.get_api_key()
        self.account_no = account_no or (os.getenv('KIS_CANO') + os.getenv('KIS_ACNT_PRDT_CD'))

    def get_positions(self, date: dtm.date) -> List[StockPosition]:
        target_date = get_last_trading_day(date)
        headers = {
            "Authorization": f"Bearer {self.api_key}",
        }
        params = {
            "date": target_date.strftime("%Y%m%d"),
            "brokerage": self.brokerage,
            "account_no": self.account_no,
        }
        r = requests.get("https://pyqqq.net/api/analytics/positions", headers=headers, params=params)
        if r.status_code == 404:
            self.logger.debug(f"404 request error: {r.text}")
            return []

        r.raise_for_status()

        data = r.json()
        positions = []
        for p in data['positions']:
            pos = StockPosition(
                asset_code=p["asset_code"],
                asset_name=p["asset_name"],
                quantity=p["quantity"],
                sell_possible_quantity=p["sell_possible_quantity"],
                average_purchase_price=Decimal(p["average_purchase_price"]),
                current_price=p["current_price"],
                current_value=p["current_value"],
                current_pnl=Decimal(p["current_pnl"]),
                current_pnl_value=p["current_pnl_value"],
            )
            positions.append(pos)

        return positions



from pyqqq.utils.api_client import raise_for_status, send_request
from typing import Union
import pandas as pd
import pyqqq.config as c


def get_ticker_info(symbol: str) -> Union[pd.DataFrame, None]:
    """
    종목 정보를 가져옵니다.

    Args:
        symbol (str): 종목 심볼 (ex. 'AAPL')

    Returns:
        pd.DataFrame: 종목 정보

        - exchange (str): 거래소 코드
        - order_symbol (str): 주문용 심볼
        - name (str): 종목명
        - kr_name (str): 한글 종목명
        - type (str): 종목 종류
        - reference_price (float): 기준가

    Examples:
        >>> df = get_ticker_info("AAPL")
        >>> print(df)
               exchange order_symbol       name kr_name   type  reference_price  bid_order_size  ask_order_size  tick_size
        symbol
        AAPL       NASD      NASAAPL  APPLE INC      애플  Stock           226.47               1               1          0
    """

    url = f"{c.PYQQQ_API_URL}/overseas-stock/tickers"
    params = {"symbol": symbol}

    r = send_request("GET", url, params=params)
    if r.status_code == 404:
        return None
    else:
        raise_for_status(r)

    return pd.DataFrame([r.json()]).set_index("symbol")
